﻿using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace UIThreadTest
{
    internal class UiThreadTest
    {
        const string html =
@"
        <body>
            <div> 
                <h1>Test</h1>
            </div>
            <h1>Hello HTML5...!</h1>
            <hr />
            <br>
            <h2>Very Simple</h2>
        </body>
 ";

        private Thread uiThread = null;
        private WebBrowser wb = null;
        private System.Windows.Forms.Timer timer = null;
        private bool bExitThread { get; set; } = false;

        private void threadFunc()
        {
            try
            {
                Application.ThreadException += Application_ThreadException;

                wb = new WebBrowser();
                timer = new System.Windows.Forms.Timer();

                wb.ScrollBarsEnabled = false;
                wb.ScriptErrorsSuppressed = true;
                wb.DocumentCompleted += Wb_DocumentCompleted;

                timer.Interval = 5000;
                timer.Tick += Timer_Tick;
                timer.Start();

                Application.Run();
            }
            catch (Exception ex)
            {
                MessageBox.Show($"threadFunc catch block \n\n {ex}");
            }
            finally
            {
                MessageBox.Show("threadFunc finally block");
                wb.Dispose();
                wb = null;
                uiThread = null;
            }
        }

        private void Application_ThreadException(object sender, ThreadExceptionEventArgs e)
        {
            MessageBox.Show($"Application_ThreadException \n\n {e.Exception}");
        }

        private void Timer_Tick(object sender, EventArgs e)
        {
            //wb.Dispose();
            wb.DocumentText = html;

            //throw new Exception("Timer_Tick User Throw Exception");

            if (bExitThread)
                Application.Exit();
        }

        private void Wb_DocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
        {
            try
            {
                wb.Width = wb.Document.Body.ScrollRectangle.Width;
                wb.Height = wb.Document.Body.ScrollRectangle.Height;

                //throw new Exception("Wb_DocumentCompleted User Throw Exception");

                using (Bitmap bitmap = new Bitmap(wb.Width, wb.Height))
                {
                    wb.DrawToBitmap(bitmap, new Rectangle(0, 0, wb.Width, wb.Height));
                    bitmap.Save(@"C:\Temp1\image.png", System.Drawing.Imaging.ImageFormat.Png);
                }
            }
            catch (System.Runtime.InteropServices.ExternalException ex)
            {
                throw;
            }
            catch (Exception ex)
            {
                throw;
            }
        }

        public void StartUIThread()
        {
            if(uiThread == null)
            {
                uiThread = new Thread(threadFunc);
                
                uiThread.SetApartmentState(ApartmentState.STA);
                //uiThread.IsBackground = true;
                uiThread.Start();
            }
        }
        
        public void Join(int milisecondsTimeout)
        {
            if (uiThread == null || uiThread.ThreadState == ThreadState.Unstarted)
                return;

            uiThread.IsBackground = false;
            bExitThread = true;

            bool timeoutState = uiThread.Join(milisecondsTimeout);
        }
    }
}
